@@ -24,6 +24,7 @@
 
 static const char description[] = I18N_NOOP("Jukebox and music manager for KDE");
 static const char scott[]       = I18N_NOOP("Author, chief dork and keeper of the funk");
+static const char michael[]     = I18N_NOOP("Assistant super-hero, fixer of many things");
 static const char daniel[]      = I18N_NOOP("System tray docking, \"inline\" tag editing,\nbug fixes, evangelism, moral support");
 static const char tim[]         = I18N_NOOP("GStreamer port");
 static const char stefan[]      = I18N_NOOP("Global keybindings support");
@@ -33,8 +34,9 @@ static const char zack[]        = I18N_NOOP("More automagical things, now using
 static const char adam[]        = I18N_NOOP("Co-conspirator in MusicBrainz wizardry");
 static const char matthias[]    = I18N_NOOP("Friendly, neighborhood aRts guru");
 static const char maks[]        = I18N_NOOP("Making JuK friendlier to people with\nterabytes of music");
-static const char michael[]     = I18N_NOOP("Bugfixes, K3b integration, endurer of pedantry");
 static const char antonio[]     = I18N_NOOP("DCOP interface");
+static const char allan[]       = I18N_NOOP("FLAC and MPC support");
+static const char nathan[]      = I18N_NOOP("Album cover manager");
 
 static KCmdLineOptions options[] =
 {
@@ -45,11 +47,12 @@ static KCmdLineOptions options[] =
 int main(int argc, char *argv[])
 {
     KAboutData aboutData("juk", I18N_NOOP("JuK"),
-                         "2.1.2", description, KAboutData::License_GPL,
-                         "(c) 2002 - 2004, Scott Wheeler", 0,
+                         "2.2", description, KAboutData::License_GPL,
+                         "(c) 2002 - 2005, Scott Wheeler", 0,
                          "http://developer.kde.org/~wheeler/juk.html");
 
     aboutData.addAuthor("Scott Wheeler", scott, "wheeler@kde.org");
+    aboutData.addAuthor("Michael Pyne", michael, "michael.pyne@kdemail.net");
     aboutData.addCredit("Daniel Molkentin", daniel, "molkentin@kde.org");
     aboutData.addCredit("Tim Jansen", tim, "tim@tjansen.de");
     aboutData.addCredit("Stefan Asserhäll", stefan, "stefan.asserhall@telia.com");
@@ -60,7 +63,8 @@ int main(int argc, char *argv[])
     aboutData.addCredit("Matthias Kretz", matthias, "kretz@kde.org");
     aboutData.addCredit("Maks Orlovich", maks, "maksim@kde.org");
     aboutData.addCredit("Antonio Larrosa Jimenez", antonio, "larrosa@kde.org");
-    aboutData.addCredit("Michael Pyne", michael, "michael.pyne@kdemail.net");
+    aboutData.addCredit("Allan Sandfeld Jensen", allan, "kde@carewolf.com");
+    aboutData.addCredit("Nathan Toone", nathan, "nathan@toonetown.com");
 
     KCmdLineArgs::init(argc, argv, &aboutData);
     KCmdLineArgs::addCmdLineOptions(options);
@@ -73,7 +77,7 @@ int main(int argc, char *argv[])
     // before JuK has finished its initialization.
 
     a.dcopClient()->suspend();
-    JuK *juk = new JuK();
+    JuK *juk = new JuK;
     a.dcopClient()->resume();
 
     a.setMainWidget(juk);
