@@ -14,10 +14,14 @@
  ***************************************************************************/
 
 #include <kiconloader.h>
+#include <kstandarddirs.h>
 #include <kdebug.h>
 
 #include <qpixmap.h>
 #include <qpainter.h>
+#include <qfile.h>
+#include <qdir.h>
+#include <qdatastream.h>
 
 #include "viewmode.h"
 #include "playlistbox.h"
@@ -249,7 +253,7 @@ void CompactViewMode::updateHeights()
 ////////////////////////////////////////////////////////////////////////////////
 
 TreeViewMode::TreeViewMode(PlaylistBox *b) : CompactViewMode(b),
-    m_canDeletePlaylists(true)
+    m_treeViewItems(5003, false), m_canDeletePlaylists(true)
 {
 
 }
@@ -288,17 +292,17 @@ void TreeViewMode::slotRemoveItem(const QString &item, unsigned column)
 {
     QString itemKey;
     if(column == PlaylistItem::ArtistColumn)
-        itemKey = "artists" + item.lower();
+        itemKey = "artists" + item;
     else if(column == PlaylistItem::GenreColumn)
-        itemKey = "genres" + item.lower();
+        itemKey = "genres" + item;
     else if(column == PlaylistItem::AlbumColumn)
-        itemKey = "albums" + item.lower();
+        itemKey = "albums" + item;
     else {
         kdWarning() << k_funcinfo << "Unhandled column type " << column << endl;
         return;
     }
 
-    if(!m_treeViewItems.contains(itemKey))
+    if(!m_treeViewItems.find(itemKey))
         return;
 
     TreeViewItemPlaylist *itemPlaylist = m_treeViewItems[itemKey];
@@ -313,12 +317,9 @@ void TreeViewMode::slotRemoveItem(const QString &item, unsigned column)
     emit signalPlaylistDestroyed(itemPlaylist);
 }
 
-void TreeViewMode::slotAddItem(const QString &item, unsigned column)
+void TreeViewMode::slotAddItems(const QStringList &items, unsigned column)
 {
-    QValueList<int> columns;
-    columns.append(column);
-
-    QString itemKey, searchCategory;
+    QString searchCategory;
     if(column == PlaylistItem::ArtistColumn)
         searchCategory = "artists";
     else if(column == PlaylistItem::GenreColumn)
@@ -330,28 +331,38 @@ void TreeViewMode::slotAddItem(const QString &item, unsigned column)
         return;
     }
 
-    itemKey = searchCategory + item.lower();
-
-    if(m_treeViewItems.contains(itemKey))
-        return;
-        
-    PlaylistSearch::ComponentList components;
+    QValueList<int> columns;
+    columns.append(column);
 
     PlaylistSearch::Component::MatchMode mode = PlaylistSearch::Component::ContainsWord;
     if(column != PlaylistItem::ArtistColumn)
-        mode = PlaylistSearch::Component::Exact;
-
-    components.append(PlaylistSearch::Component(item, false, columns, mode));
+	mode = PlaylistSearch::Component::Exact;
 
+    PlaylistSearch::ComponentList components;
     PlaylistList playlists;
     playlists.append(CollectionList::instance());
 
-    PlaylistSearch s(playlists, components, PlaylistSearch::MatchAny, false);
+    QString itemKey, item;
 
-    TreeViewItemPlaylist *p = new TreeViewItemPlaylist(playlistBox(), s, item, false);
-    playlistBox()->setupPlaylist(p, "midi", m_searchCategories[searchCategory]);
+    PlaylistBox::Item *itemParent = m_searchCategories[searchCategory];
 
-    m_treeViewItems[itemKey] = p;
+    for(QStringList::ConstIterator it = items.begin(); it != items.end(); ++it) {
+	item = *it;
+	itemKey = searchCategory + item;
+
+	if(m_treeViewItems.find(itemKey))
+	    continue;
+	    
+	components.clear();
+	components.append(PlaylistSearch::Component(item, false, columns, mode));
+
+	PlaylistSearch s(playlists, components, PlaylistSearch::MatchAny, false);
+
+	TreeViewItemPlaylist *p = new TreeViewItemPlaylist(playlistBox(), s, item, false);
+	playlistBox()->setupPlaylist(p, "midi", itemParent);
+
+	m_treeViewItems.insert(itemKey, p);
+    }
 }
 
 void TreeViewMode::slotCanDeletePlaylist(bool canDelete)
@@ -386,9 +397,6 @@ void TreeViewMode::setupCategories()
 
     i = new PlaylistBox::Item(collectionItem, "cdimage", i18n("Genres"));
     m_searchCategories.insert("genres", i);
-
-    for(QDictIterator<PlaylistBox::Item> it(m_searchCategories); it.current(); ++it)
-        it.current()->setSortedFirst(true);
 }
 
 #include "viewmode.moc"
