@@ -21,6 +21,7 @@
 #ifndef ENCODEFILEIMP_H
 #define ENCODEFILEIMP_H
 
+#include "libkcddb/genres.h"
 #include "encodefile.h"
 #include "qmap.h"
 
@@ -38,7 +39,7 @@ signals:
   void startJob(Job *newJob);
 
 public:
-  EncodeFileImp( QMap<QString, QString> genres, QWidget* parent = 0, const char* name = 0);
+  EncodeFileImp(QWidget* parent = 0, const char* name = 0);
 
 protected slots:
   // Encode button
@@ -46,7 +47,7 @@ protected slots:
 
 private:
   // List of genres and i18n versions
-  QMap<QString, QString> genres;
+  KCDDB::Genres m_genres;
 
 };
 
