@@ -5,7 +5,7 @@
  * Copyright (C) 2001 Adrian Schroeter <adrian@suse.de>
  * Copyright (C) 2003 Richard Lärkäng <richard@goteborg.utfors.se>
  * Copyright (C) 2003 Scott Wheeler <wheeler@kde.org>
- * Copyright (C) 2004 Benjamin Meyer <ben + audiocd at meyerhome dot net>
+ * Copyright (C) 2004, 2005 Benjamin Meyer <ben + audiocd at meyerhome dot net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,8 +60,9 @@ extern "C"
 #include <sys/cdio.h>
 #endif
 
+#include <kdemacros.h>
 void paranoiaCallback(long, int);
-	int kdemain(int argc, char ** argv);
+KDE_EXPORT int kdemain(int argc, char ** argv);
 #ifndef CDPARANOIA_STATIC
 	int FixupTOC(cdrom_drive *d, int tracks);
 #endif
@@ -81,6 +82,7 @@ void paranoiaCallback(long, int);
 #include <kmacroexpander.h>
 #include <qfile.h>
 #include <qfileinfo.h>
+#include <kcmdlineargs.h>
 #include <kdebug.h>
 #include <kprotocolmanager.h>
 #include <klocale.h>
@@ -195,22 +197,31 @@ long my_first_sector(cdrom_drive *drive)
 
 using namespace AudioCD;
 
+static const KCmdLineOptions options[] =
+{
+    { "+protocol", I18N_NOOP("Protocol name"), 0 },
+    { "+pool", I18N_NOOP("Socket name"), 0 },
+    { "+app", I18N_NOOP("Socket name"), 0 },
+    KCmdLineLastOption
+};
+
 int
 kdemain(int argc, char ** argv)
 {
 	// KApplication is used as libkcddb uses ioslaves which need a valid
 	// kapp pointer
-	KApplication app(argc, argv, "kio_audiocd", false, true);
+	// GUIenabled must be true as libkcddb sometimes wants to communicate
+	// with the user
+	putenv(strdup("SESSION_MANAGER="));
+	KApplication::disableAutoDcopRegistration();
+	KCmdLineArgs::init(argc, argv, "kio_audiocd", 0, 0, 0, 0);
+	KCmdLineArgs::addCmdLineOptions(options);
+	KApplication app(false, true);
 
 	kdDebug(7117) << "Starting " << getpid() << endl;
 
-	if (argc != 4)
-	{
-		 kdError(7117) << "Usage: kio_audiocd protocol domain-socket1 domain-socket2" << endl;
-		 exit(-1);
-	}
-
-	AudioCDProtocol slave(argv[2], argv[3]);
+	KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
+	AudioCDProtocol slave(args->arg(0), args->arg(1), args->arg(2));
 	slave.dispatchLoop();
 
 	kdDebug(7117) << "Done" << endl;
@@ -288,8 +299,8 @@ public:
 int paranoia_read_limited_error;
 
 
-AudioCDProtocol::AudioCDProtocol (const QCString & pool, const QCString & app)
-	: SlaveBase("audiocd", pool, app)
+AudioCDProtocol::AudioCDProtocol(const QCString & protocol, const QCString & pool, const QCString & app)
+	: SlaveBase(protocol, pool, app)
 {
 	d = new Private;
 
@@ -363,7 +374,7 @@ struct cdrom_drive * AudioCDProtocol::initRequest(const KURL & url)
 	// Determine what file or folder that is wanted.
 	//
 
-	d->fname = url.filename(false);
+	d->fname = url.fileName(false);
 	QString dname = url.directory(true, false);
 	if (!dname.isEmpty() && dname[0] == '/')
 		dname = dname.mid(1);
@@ -707,6 +718,7 @@ static void app_dir(UDSEntry& e, const QString & n, size_t s)
 	app_entry(e, KIO::UDS_FILE_TYPE, S_IFDIR);
 	app_entry(e, KIO::UDS_ACCESS, 0400);
 	app_entry(e, KIO::UDS_SIZE, s);
+	app_entry(e, KIO::UDS_MIME_TYPE, "inode/directory");
 }
 
 static void app_file(UDSEntry& e, const QString & n, size_t s)
@@ -901,7 +913,12 @@ struct cdrom_drive *AudioCDProtocol::pickDrive()
 				error(KIO::ERR_CANNOT_OPEN_FOR_READING, d->device);
 			else if(!fi.isWritable())
 				error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, d->device);
+#if KDE_IS_VERSION (3, 3, 89)
+			else error(KIO::ERR_SLAVE_DEFINED, 
+i18n("Unknown error.  If you have a cd in the drive try running cdparanoia -vsQ as yourself (not root). Do you see a track list? If not, make sure you have permission to access the CD device. If you are using SCSI emulation (possible if you have an IDE CD writer) then make sure you check that you have read and write permissions on the generic SCSI device, which is probably /dev/sg0, /dev/sg1, etc.. If it still does not work, try typing audiocd:/?device=/dev/sg0 (or similar) to tell kio_audiocd which device your CD-ROM is."));
+#else			
 			else error(KIO::ERR_UNKNOWN, d->device);
+#endif
 		}
 	}
 
