@@ -1,7 +1,7 @@
 /*
  * KMix -- KDE's full featured mini mixer
  *
- * $Id: kmix.cpp,v 1.185.2.4 2004/11/23 21:07:47 helio Exp $
+ * $Id: kmix.cpp,v 1.197 2005/02/08 20:36:58 esken Exp $
  *
  * Copyright (C) 2000 Stefan Schimanski <schimmi@kde.org>
  * Copyright (C) 2001 Preston Brown <pbrown@kde.org>
@@ -73,7 +73,7 @@ KMixWindow::KMixWindow()
 	m_isVisible = false;
 	m_mixerWidgets.setAutoDelete(true);
 	loadConfig(); // Need to load config before initMixer(), due to "MultiDriver" keyword
-	MixerToolBox::initMixer(m_mixers, m_multiDriverMode);
+	MixerToolBox::initMixer(m_mixers, m_multiDriverMode, m_hwInfoString);
 	initActions();
 	initWidgets();
 	initMixerWidgets();
@@ -113,10 +113,12 @@ KMixWindow::initActions()
 	KStdAction::preferences( this, SLOT(showSettings()), actionCollection());
 	KStdAction::keyBindings( guiFactory(), SLOT(configureShortcuts()), actionCollection());
 
+/* Belongs in KMixDockWidget
 	(void)new KToggleAction( i18n( "M&ute" ), 0, this, SLOT( dockMute() ),
 				 actionCollection(), "dock_mute" );
-
+*/
 	(void) new KAction( i18n( "Hardware &Information" ), 0, this, SLOT( slotHWInfo() ), actionCollection(), "hwinfo" );
+	(void) new KAction( i18n( "Hide Mixer Window" ), Key_Escape, this, SLOT(hide()), actionCollection(), "hide_kmixwindow" );
 	createGUI( "kmixui.rc" );
 }
 
@@ -187,30 +189,26 @@ KMixWindow::updateDocking()
                 // !! This should be a View in the future
 		m_dockWidget = new KMixDockWidget( m_mixers.first(), this, "mainDockWidget", m_volumeWidget );
 
+/* Belongs in KMixDockWidget
 		// create RMB menu
 		KPopupMenu *menu = m_dockWidget->contextMenu();
 
 		// !! check this
 		KAction *a = actionCollection()->action( "dock_mute" );
 		if ( a ) a->plug( menu );
+*/
+
+		/*
+		 * Mail from 31.1.2005: "make sure your features are at least string complete"
+		 * Preparation for fixing Bug #55078 - scheduled for KDE3.4.1 .
+		 * This text will be plugged into the dock-icon popup menu.
+		 */
+		QString selectChannel = i18n("Select Channel"); // This text will be used in KDE3.4.1 !!!
 
 		m_dockWidget->show();
 	}
 }
 
-void
-KMixWindow::dockMute()
-{
-	Mixer *mixer = m_mixers.first();
-        if ( !mixer )
-            return;
-	MixDevice *masterDevice = ( *mixer )[ mixer->masterDevice() ];
-
-	masterDevice->setMuted( !masterDevice->isMuted() );
-	mixer->writeVolumeToHW( masterDevice->num(), masterDevice->getVolume() );
-
-}
-
 void
 KMixWindow::saveSettings()
 {
@@ -505,7 +503,7 @@ void
 KMixWindow::showEvent( QShowEvent * )
 {
     if ( m_visibilityUpdateAllowed )
-	m_isVisible = true;
+	m_isVisible = isVisible();
     // !! could possibly start polling now (idea: use someting like ref() and unref() on Mixer instance
 }
 
@@ -514,7 +512,7 @@ KMixWindow::hideEvent( QHideEvent * )
 {
     if ( m_visibilityUpdateAllowed )
     {
-	m_isVisible = false;
+	m_isVisible = isVisible();
     }
     // !! could possibly stop polling now (idea: use someting like ref() and unref() on Mixer instance
     //    Update: This is a stupid idea, because now the views are responsible for updating. So it will be done in the Views.
