@@ -3,6 +3,7 @@
  *
  *
  * Copyright (C) 2000 Stefan Schimanski <1Stein@gmx.de>
+ * Copyright (C) 2003 Sven Leiber <s.leiber@web.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,40 +17,60 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #ifndef KMIXDOCKWIDGET_H
 #define KMIXDOCKWIDGET_H
 
 #include <qwidget.h>
+#include <qvbox.h>
 #include <ksystemtray.h>
 
 class Mixer;
 class QFrame;
+class KAudioPlayer;
+class ViewDockAreaPopup;
+class MixDeviceWidget;
+class Volume;
 
 class KMixDockWidget : public KSystemTray  {
    Q_OBJECT
 
    friend class KMixWindow;
 
- public: 
-   KMixDockWidget(Mixer *, QWidget *parent=0, const char *name=0);
+ public:
+   KMixDockWidget(Mixer *, QWidget *parent=0, const char *name=0, bool volumePopup=true);
    ~KMixDockWidget();
 
- protected slots:  
-   void setVolumeTip(int, Volume);
+   void setErrorPixmap();
+   void ignoreNextEvent();
+   ViewDockAreaPopup* getDockAreaPopup();
+
+	Mixer *m_mixer;
+   ViewDockAreaPopup *_dockAreaPopup;
+   KAudioPlayer *_audioPlayer;
+
+ public slots:
+   void setVolumeTip();
+   void updatePixmap();
 
  protected:
    void createMasterVolWidget();
    void mousePressEvent(QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);
-   void mouseDoubleClickEvent(QMouseEvent *);
+   void wheelEvent(QWheelEvent *);
    void contextMenuAboutToShow( KPopupMenu* menu );
+   void toggleMinimizeRestore();
 
  private:
-   Mixer *m_mixer;
-   QFrame *masterVol;   
+   bool _playBeepOnVolumeChange;
+   bool _ignoreNextEvent;
+   int  _oldToolTipValue;
+   char _oldPixmapType;
+   bool _volumePopup;
+ private slots:
+   void dockMute();
 };
 
 #endif
