@@ -3,6 +3,7 @@
  *
  * Copyright (c) 1997 Bernd Johannes wuebben@math.cornell.edu
  * Copyright (c) 2002-2003 Aaron J. Seigo <aseigo@kde.org>
+ * Copyright (c) 2004 Alexander Kern <alex.kern@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,11 +28,13 @@
 #include <qhbox.h>
 #include <qvbox.h>
 #include <qapplication.h>
+#include <qgroupbox.h>
 
 #include <dcopclient.h>
 #include <kaboutdata.h>
 #include <kaccel.h>
 #include <kaction.h>
+#include <dcopref.h>
 #include <kcharsets.h>
 #include <kcmdlineargs.h>
 #include <kconfig.h>
@@ -42,6 +45,7 @@
 #include <khelpmenu.h>
 #include <kkeydialog.h>
 #include <kiconloader.h>
+#include <kinputdialog.h>
 #include <klocale.h>
 #include <kmainwindow.h>
 #include <kmessagebox.h>
@@ -60,7 +64,6 @@
 
 #include "docking.h"
 #include "kscd.h"
-#include "mgconfdlg.h"
 #include "version.h"
 #include "prefs.h"
 
@@ -72,14 +75,19 @@
 extern "C" {
     // We don't have libWorkMan installed already, so get everything
     // from within our own directory
+#include "libwm/include/wm_cddb.h"
 #include "libwm/include/wm_cdrom.h"
 #include "libwm/include/wm_cdtext.h"
 #include "libwm/include/wm_config.h"
+#include "libwm/include/wm_cdinfo.h"
 }
 
 #include "cddbdlg.h"
 #include "configWidget.h"
 #include "kvolumecontrol.h"
+#include <qtextcodec.h>
+
+#include <fixx11h.h>
 
 #define N_TRACK_FIRST   1
 #define N_TRACK_STOP 0
@@ -110,15 +118,14 @@ KSCD::KSCD( QWidget *parent, const char *name )
     jumpToTrack(0L),
     cddrive_is_ok(true),
     have_new_cd(true),
-    //updateDialog(false), //!!!!
     updateTime(true),
-    cddb(0),
-    revision(0), // The first freedb revision is "0" //!!!!
-    year(0),
     m_dockWidget(0)
 {
+  cddbInfo.clear(); // The first freedb revision is "0" //!!!!
   random_current      = random_list.begin();
 
+  cddb = new KCDDB::Client();
+
 #if defined(BUILD_CDDA)
   audio_systems_list
                      << "arts"
@@ -189,7 +196,10 @@ KSCD::KSCD( QWidget *parent, const char *name )
   action = new KAction(i18n("Loop"), Key_L, this, SLOT(loopClicked()), m_actions, "Loop");
   action = new KAction(i18n("Eject"), CTRL + Key_E, this, SLOT(ejectClicked()), m_actions, "Eject");
   action = new KAction(i18n("Increase Volume"), Key_Plus, this, SLOT(incVolume()), m_actions, "IncVolume");
-  action = new KAction(i18n("Increase Volume"), Key_Equal, this, SLOT(incVolume()), m_actions, "IncVolume Alt");
+	KShortcut increaseVolume = action->shortcut();
+	increaseVolume.append( KKey( Key_Equal ) );
+	action->setShortcut( increaseVolume );
+
   action = new KAction(i18n("Decrease Volume"), Key_Minus, this, SLOT(decVolume()), m_actions, "DecVolume");
   action = new KAction(i18n("Options"), CTRL + Key_T, this, SLOT(showConfig()), m_actions, "Options");
   action = new KAction(i18n("Shuffle"), Key_R, this, SLOT(randomSelected()), m_actions, "Shuffle");
@@ -197,6 +207,8 @@ KSCD::KSCD( QWidget *parent, const char *name )
 
   setupPopups();
 
+  m_actions->readShortcutSettings("Shortcuts");
+
   if (Prefs::looping())
   {
     loopled->on();
@@ -206,8 +218,6 @@ KSCD::KSCD( QWidget *parent, const char *name )
 
   setDocking(Prefs::docking());
 
-  connectDCOPSignal(0, 0, "KDE_emailSettingsChanged()", "emailSettingsChanged()", false);
-
   setFocusPolicy(QWidget::NoFocus);
 
   songListCB->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
@@ -260,13 +270,17 @@ void KSCD::setVolume(int v)
     volumeSlider->setValue(v);
 }
 
+void KSCD::setDevice(const QString& dev) {
+    Prefs::self()->setCdDevice(dev);
+    setDevicePaths();
+}
+
 /**
- * Initialize smallfont which fits into the 13 and 14 pixel widgets
- * and verysmallfont which is slightly smaller.
+ * Initialize smallfont which fits into the 13 and 14 pixel widgets.
  */
 void KSCD::initFont()
 {
-  int theSmallPtSize = 10;
+/*  int theSmallPtSize = 10;
 
   // Find a font that fits the 13 and 14 pixel widgets
   QFont fn( KGlobalSettings::generalFont().family(), theSmallPtSize, QFont::Bold );
@@ -282,11 +296,8 @@ void KSCD::initFont()
           fits = true;
       }
   }
-
   smallfont = QFont(KGlobalSettings::generalFont().family(), theSmallPtSize, QFont::Bold);
-  verysmallfont = QFont(KGlobalSettings::generalFont().family(),
-                        (theSmallPtSize > 4) ? theSmallPtSize - 2 : 2,
-                        QFont::Bold);
+*/
 } // initFont()
 
 /**
@@ -297,7 +308,7 @@ void KSCD::drawPanel()
 {
   setIcons();
   adjustSize();
-  
+
   const int D = 6;
   for (int u = 0; u < 5; u++) {
      trackTimeLED[u] = new BW_LED_Number(frameleds);
@@ -309,7 +320,7 @@ void KSCD::drawPanel()
 
   QString zeros("--:--");
   setLEDs(zeros);
-  
+
   queryled = new LedLamp(symbols);
   queryled->move(symbols->width()-20, D + 1);
   queryled->off();
@@ -465,7 +476,7 @@ void KSCD::stopClicked()
 
     kapp->processEvents();
     kapp->flushX();
-    wm_cd_stop(); // must be first, apparently
+    wm_cd_stop(); // we need some error handling here!! it does return a useful value
 } // stopClicked()
 
 void KSCD::prevClicked()
@@ -600,51 +611,6 @@ void KSCD::quitClicked()
     kapp->quit();
 } // quitClicked()
 
-
-void KSCD::closeEvent( QCloseEvent *e )
-{
-    // we need to figure out if we were called by the system tray
-    // to decide whether or not to actually quit or not =/
-    // this behaviour of ksystemtray is, IMHO, very silly
-    const QObject* caller = sender();
-    while (m_dockWidget && caller)
-    {
-        if (caller == m_dockWidget)
-        {
-            break;
-        }
-        caller = caller->parent();
-    }
-
-    if (Prefs::docking() && !caller && !kapp->sessionSaving())
-    {
-        hide();
-        e->ignore();
-        return;
-    }
-
-    /* Stop playing the CD */
-    int cur_cdmode = wm_cd_status();
-    if ( Prefs::stopExit() && cur_cdmode == WM_CDM_PLAYING )
-        wm_cd_stop();
-
-    writeSettings();
-    //setShuffle(0);
-
-    statuslabel->clear();
-
-    setLEDs( "--:--" );
-
-    kapp->processEvents();
-    kapp->flushX();
-
-    // TODO: is this really necessary given the stopClicked() above?
-    if (Prefs::stopExit())
-        wm_cd_stop();
-
-     e->accept();
-} // closeEvent
-
 bool KSCD::event( QEvent *e )
 {
     return QWidget::event(e);
@@ -703,18 +669,25 @@ void KSCD::ejectClicked()
       statuslabel->setText(i18n("Ejecting"));
       kapp->processEvents();
       kapp->flushX();
-      setArtist("");
       setTitle(0);
-      tracktitlelist.clear();
-      extlist.clear();
-      category = "";
-      genre = "";
+      cddbInfo.clear();
+      populateSongList("");
 
       wm_cd_stop();
       wm_cd_eject();
     }
 } // ejectClicked
 
+void KSCD::closeEvent(QCloseEvent *e)
+{
+    if (Prefs::docking() && !kapp->sessionSaving())
+    {
+        hide();
+        e->ignore();
+        return;
+    }
+}
+
 void KSCD::randomSelected()
 {
     setShuffle(Prefs::randomPlay()?0:1);
@@ -767,7 +740,7 @@ void KSCD::showConfig()
     confWidget = new configWidget(this, 0, "Kscd");
 
     // kscd config page
-    configDialog -> addPage(confWidget, i18n("CD Player"), "kscd", i18n("Settings & Behavior"));
+    configDialog->addPage(confWidget, i18n("CD Player"), "kscd", i18n("Settings & Behavior"));
 
     // libkcddb page
     KService::Ptr libkcddb = KService::serviceByDesktopName("libkcddb");
@@ -780,7 +753,10 @@ void KSCD::showConfig()
             if (m)
             {
                 m->load();
-                configDialog -> addPage(m, QString("freedb"), "cdtrack", i18n("Configure Fetching Items"));
+                KCDDB::Config* cfg = new KCDDB::Config();
+                cfg->readConfig();
+                configDialog -> addPage(m, cfg, QString("CDDB"), "cdtrack", i18n("Configure Fetching Items"));
+
                 connect(configDialog, SIGNAL(okClicked()), m, SLOT(save()));
                 connect(configDialog, SIGNAL(applyClicked()), m, SLOT(save()));
                 connect(configDialog, SIGNAL(defaultClicked()), m, SLOT(defaults()));
@@ -813,28 +789,42 @@ void KSCD::configureKeys()
 void KSCD::setDevicePaths()
 {
     cddrive_is_ok = false;
+    KURL deviceUrl(Prefs::cdDevice());
+    kdDebug() << "Current device is " << deviceUrl.url() << endl;
+    if (deviceUrl.protocol()=="media")
+    {
+    	kdDebug(67000) << "Asking mediamanager for " << deviceUrl.path(-1).mid(1) << endl;
+        DCOPRef mediamanager("kded","mediamanager");
+	// mediamanager does not like paths with leading /
+        DCOPReply rep = mediamanager.call("properties(QString)",deviceUrl.path(-1).mid(1));
+	if (!rep.isValid()) deviceUrl.setPath(DEFAULT_CD_DEVICE);
+	else {
+		QStringList properties = rep;
+		kdDebug(67000) << "Got properties " << properties << endl;
+		deviceUrl.setPath(properties[5]);
+	      }
+    }
+
     int ret = wm_cd_init(
 #if defined(BUILD_CDDA)
         (Prefs::digitalPlayback())?WM_CDDA:WM_CDIN,
-        QFile::encodeName(Prefs::cdDevice()),
+        QFile::encodeName(deviceUrl.path(-1)),
         (Prefs::digitalPlayback())?Prefs::audioSystem().ascii():"",
         (Prefs::digitalPlayback())?Prefs::audioDevice().ascii():"",
         0);
     kdDebug(67000) << "Device changed to "
         << ((Prefs::digitalPlayback())?"WM_CDDA, ":"WM_CDIN, ")
-        << Prefs::cdDevice() << ", "
+        << deviceUrl.path() << ", "
         << ((Prefs::digitalPlayback())?Prefs::audioSystem():"") << ", "
         << ((Prefs::digitalPlayback())?Prefs::audioDevice():"") << ". returns status " << ret << "\n";
 #else
-        WM_CDIN, QFile::encodeName(Prefs::cdDevice()), 0, 0, 0);
-    kdDebug(67000) << "Device changed to " << Prefs::cdDevice() << ". returns status  " << ret << "\n";
+        WM_CDIN, QFile::encodeName(deviceUrl.path(-1)), 0, 0, 0);
+    kdDebug(67000) << "Device changed to " << deviceUrl.path() << ". returns status  " << ret << "\n";
 #endif
 
-    setArtist("");
     setTitle(0);
-    tracktitlelist.clear();
-    extlist.clear();
-    clearSongList();
+    cddbInfo.clear();
+    populateSongList("");
     cddrive_is_ok = true;
     initCDROM();
 } // setDevicePath()
@@ -847,6 +837,7 @@ void KSCD::setDocking(bool dock)
         if (!m_dockWidget)
         {
             m_dockWidget = new DockWidget(this, "dockw");
+            connect(m_dockWidget, SIGNAL(quitSelected()), this, SLOT(quitClicked()));
         }
 
         m_dockWidget->show();
@@ -1019,7 +1010,25 @@ void KSCD::cdMode()
         prev_track = -1;
     } else if(have_new_cd) {
         have_new_cd = false;
-	get_cddb_info(false);
+
+        if (!cd ||
+           !cddrive_is_ok ||
+           wm_cd_status() < 1)
+        {
+          kdDebug(67000) << "CD pointer invalid cd " << cd << " cddrive_is_ok " << cddrive_is_ok << endl;
+          cddb_failed(CDDB::NoRecordFound);
+          return;
+        }
+
+        // Don't crash if no disc is in
+        if( cd->length == 0 ) {
+          kdDebug(67000) << "CD length seems to be zero" << endl;
+          cddb_failed(CDDB::NoRecordFound);
+          return;
+        }
+
+        lookupDevice();
+        lookupCDDB();
 
         if(Prefs::autoplay() && cdmode == WM_CDM_STOPPED)
             playClicked();
@@ -1113,12 +1122,10 @@ void KSCD::cdModeChanged(int previous, int cdmode)
                 statuslabel->setText(i18n("Ejected"));
             }
 
-            tracktitlelist.clear();
-            extlist.clear();
-            clearSongList();
+            cddbInfo.clear();
+            populateSongList("");
             totaltimelabel->clear();
             totaltimelabel->lower();
-            setArtist("");
 
             updateDisplayedTrack(N_TRACK_UNKNOW);
             break;
@@ -1150,7 +1157,12 @@ void KSCD::setColors()
 {
     QColor led_color = Prefs::ledColor();
     QColor background_color = Prefs::backColor();
+
     backdrop->setBackgroundColor(background_color);
+    frameoben->setBackgroundColor(background_color);
+    frameleds->setBackgroundColor(background_color);
+    symbols->setBackgroundColor(background_color);
+    frameartisttitle->setBackgroundColor(background_color);
 
     QColorGroup colgrp( led_color, background_color, led_color,led_color , led_color,
                         led_color, white );
@@ -1170,12 +1182,12 @@ void KSCD::setColors()
         trackTimeLED[u]->setLEDColor(led_color,background_color);
     }
 
-    titlelabel ->setFont( smallfont );
-    artistlabel->setFont( smallfont );
-    volumelabel->setFont( smallfont );
-    statuslabel->setFont( smallfont );
-    tracklabel ->setFont( smallfont );
-    totaltimelabel->setFont( smallfont );
+    titlelabel ->setFont( Prefs::ledFont() );
+    artistlabel->setFont( Prefs::ledFont() );
+    volumelabel->setFont( Prefs::ledFont() );
+    statuslabel->setFont( Prefs::ledFont() );
+    tracklabel ->setFont( Prefs::ledFont() );
+    totaltimelabel->setFont( Prefs::ledFont() );
 }
 
 void KSCD::readSettings()
@@ -1213,10 +1225,8 @@ void KSCD::CDDialogSelected()
     {
         cddialog = new CDDBDlg(this);
 
-        cddialog->setData(cd,tracktitlelist,extlist,xmcd_data,category, genre,
-                        revision,year,playlist,pathlist);
-
-        connect(cddialog,SIGNAL(cddbQuery(bool)),SLOT(get_cddb_info(bool)));
+        cddialog->setData(cddbInfo, trackStartFrames, playlist);
+        connect(cddialog,SIGNAL(cddbQuery()),SLOT(lookupCDDB()));
         connect(cddialog,SIGNAL(finished()),SLOT(CDDialogDone()));
         connect(cddialog,SIGNAL(play(int)),SLOT(trackSelected(int)));
     }
@@ -1231,24 +1241,9 @@ void KSCD::CDDialogDone()
   cddialog = 0L;
 }
 
-void KSCD::get_cddb_info(bool /*_updateDialog*/)
+void KSCD::lookupCDDB()
 {
-    kdDebug(67000) << "get_cddb_info() called" << endl;
-    if (!cd ||
-        !cddrive_is_ok ||
-        wm_cd_status() < 1)
-    {
-        kdDebug(67000) << "CD pointer invalid cd " << cd << " cddrive_is_ok " << cddrive_is_ok << endl;
-        cddb_no_info();
-        return;
-    }
-
-    // Don't crash if no disc is in
-    if( cd->length == 0 ) {
-        kdDebug(67000) << "CD length seems to be zero" << endl;
-        cddb_no_info();
-        return;
-    }
+    kdDebug(67000) << "lookupCDDB() called" << endl;
 
     // FIXME: what is the total time label setting stuff doing in CDDB info?!
     QTime dml;
@@ -1262,137 +1257,148 @@ void KSCD::get_cddb_info(bool /*_updateDialog*/)
 
     totaltimelabel->setText(fmt);
 
-    KCDDB::TrackOffsetList querylist;
-    tracktitlelist.clear();
-    tracktitlelist.append(i18n("Start freedb lookup."));
-    populateSongList();
+    populateSongList(i18n("Start freedb lookup."));
 
-    for(int i = 0 ; i < wm_cd_getcountoftracks(); i++)
-    {
-        querylist << cd->trk[i].start;
-    }
     setShuffle(2);
 
-    querylist << cd->trk[0].start << cd->trk[wm_cd_getcountoftracks()].start;
     led_on();
 
-    delete cddb;
-    cddb = new KCDDB::Client();
+    cddb->config().reparse();
     cddb->setBlockingMode(false);
-    connect(cddb, SIGNAL(finished(CDDB::Result)),
-            this, SLOT(cddb_done(CDDB::Result)));
+    connect(cddb, SIGNAL(finished(CDDB::Result)), this, SLOT(lookupCDDBDone(CDDB::Result)));
 
-    cddb->lookup(querylist);
-} // get_cddb_info
+    cddb->lookup(trackStartFrames);
+} // lookupCDDB
 
-void KSCD::cddb_done(CDDB::Result result)
+void KSCD::lookupCDDBDone(CDDB::Result result)
 {
-    // CDDBTODO: figure out why using CDDB::Success doesn't compile?!
-    if ((result != 0 /*KCDDB::CDDB::Success*/) &&
+    if ((result != KCDDB::CDDB::Success) &&
         (result != KCDDB::CDDB::MultipleRecordFound))
     {
-        cddb_failed();
+        cddb_failed(result);
         return;
     }
-    KCDDB::CDInfo cddbInfo = cddb->bestLookupResponse();
 
-    tracktitlelist.clear();
-    extlist.clear();
-
-    // CDDBTODO: we really should get the artist off the 'tracktitlelist'
-    tracktitlelist << cddbInfo.artist + " / " + cddbInfo.title;
-
-    revision = cddbInfo.revision;
-    category = cddbInfo.category;
-    genre = cddbInfo.genre;
-    extlist << cddbInfo.extd;
-    year = cddbInfo.year;
-
-    KCDDB::TrackInfoList::ConstIterator it(cddbInfo.trackInfoList.begin());
-    KCDDB::TrackInfoList::ConstIterator end(cddbInfo.trackInfoList.end());
-    for (; it != end; ++it)
-    {
-        tracktitlelist << (*it).title;
-        extlist << (*it).extt;
+    // The intent of the original code here seems to have been to perform the
+    // lookup, and then to convert all the string data within the CDDB response
+    // using the use Prefs::selectedEncoding() and a QTextCodec. However, that
+    // seems to be irrelevant these days.
+    KCDDB::CDInfo info = cddb->bestLookupResponse();
+    // TODO Why doesn't libcddb not return MultipleRecordFound?
+    //if( result == KCDDB::CDDB::MultipleRecordFound ) {
+    if( cddb->lookupResponse().count() > 1 ) {
+      CDInfoList cddb_info = cddb->lookupResponse();
+      CDInfoList::iterator it;
+      QStringList list;
+      uint defaultChoice = 0;
+      uint maxrev = 0;
+      uint c = 0;
+      for ( it = cddb_info.begin(); it != cddb_info.end(); ++it  ) {
+        list.append( QString("%1, %2, %3").arg((*it).artist).arg((*it).title)
+            .arg((*it).genre));
+        KCDDB::CDInfo cinfo = *it;
+        if ( ( *it ).revision >= maxrev ) {
+          maxrev = info.revision;
+          defaultChoice = c;
+        }
+        c++;
+      }
+  
+      bool ok(false);
+      QString res = KInputDialog::getItem(
+              i18n("Select CDDB Entry"),
+              i18n("Select a CDDB entry:"), list, defaultChoice, false, &ok,
+              this );
+      if ( ok ) {
+        // The user selected and item and pressed OK
+        uint c = 0;
+        for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
+          if( *it == res)  break;
+          c++;
+        }
+        if( c < cddb_info.size() )
+          info = cddb_info[c];
+      } else {
+        return;
+        // user pressed Cancel
+      }
     }
-
-    populateSongList();
+  
+    // Some sanity provisions to ensure that the number of records matches what
+    // the CD actually contains.
+    while (info.trackInfoList.count() < cddbInfo.trackInfoList.count())
+    {
+      info.trackInfoList.append(KCDDB::TrackInfo());
+    }
+    while (info.trackInfoList.count() > cddbInfo.trackInfoList.count())
+    {
+      info.trackInfoList.pop_back();
+    }
+    cddbInfo = info;
+    populateSongList("");
 
     // In case the cddb dialog is open, update it
     if (cddialog)
-      cddialog->setData(cd,tracktitlelist,extlist,xmcd_data,category, genre,
-                        revision,year,playlist,pathlist);
+      cddialog->setData(cddbInfo, trackStartFrames, playlist);
 
     led_off();
-} // cddb_done
+} // lookupCDDBDone
 
-void KSCD::get_cdtext_info(void)
+void KSCD::lookupDevice(void)
 {
-    struct cdtext_info *p_cdtext;
-    kdDebug(67000) << "get_cdtext_info() called" << endl;
-    //setArtist("");
-    //setTitle(0);
-    //songListCB->clear();
-    tracktitlelist.clear();
-    extlist.clear();
+  // Initialise the album from the CD.
+  cddbInfo.clear();
+  trackStartFrames.clear();
+  cddbInfo.id = QString::number(cddb_discid(), 16);
+  unsigned numberOfTracks = wm_cd_getcountoftracks();
+  cddbInfo.length = cd->trk[numberOfTracks].start - cd->trk[0].start;
+  struct cdtext_info *info = wm_cd_get_cdtext();
+  if (info && info->valid) {
+    cddbInfo.artist = reinterpret_cast<char*>(info->blocks[0]->name[0]);
+    cddbInfo.title = reinterpret_cast<char*>(info->blocks[0]->performer[0]);
+  }
+  else {
+    cddbInfo.artist = i18n("Unknown Artist");
+    cddbInfo.title = i18n("Unknown Album");
+  }
 
-    p_cdtext = wm_cd_get_cdtext();
-    if(p_cdtext && p_cdtext->valid) {
-        tracktitlelist.append(QString().sprintf("%s / %s", (const char*)(p_cdtext->blocks[0]->name[0]),
-            (const char*)(p_cdtext->blocks[0]->performer[0])));
-        //titlelabel->setText(QString((const char*)(p_cdtext->blocks[0]->name[1])));
-        //artistlabel->setText(tracktitlelist.first());
-
-
-        // if it's a sampler, we'll do artist/title
-        bool isSampler = (qstricmp(reinterpret_cast<char*>(p_cdtext->blocks[0]->performer[0]), "various") == 0);
-
-        int at = 1;
-        for (; at < (p_cdtext->count_of_entries); ++at) {
-            QString title;
-            if (isSampler) {
-                title.sprintf("%s / %s", p_cdtext->blocks[0]->performer[at], p_cdtext->blocks[0]->name[at]);
-            } else {
-                title = reinterpret_cast<char*>(p_cdtext->blocks[0]->name[at]);
-            }
-            tracktitlelist.append(title);
-        }
-    } else {
-        kdDebug(67000) << "cdtext invalid" << endl;
+    // If it's a sampler, we'll do artist/title.
+  bool isSampler = (cddbInfo.title.compare("Various") == 0);
+  KCDDB::TrackInfo track;
+  for (unsigned i = 1; i <= numberOfTracks; i++) {
+    if (info && info->valid) {
+      if (isSampler) {
+        track.title = reinterpret_cast<char*>(info->blocks[0]->performer[i]);
+        track.title.append(" / ");
+        track.title.append(reinterpret_cast<char*>(info->blocks[0]->name[i]));
+      } else {
+        track.title = reinterpret_cast<char*>(info->blocks[0]->name[i]);
+      }
     }
-      
-    populateSongList();
+    else {
+      track.title = QString::null;
+    }
+    // FIXME: KDE4
+    // track.length = cd->trk[i - 1].length;
+    trackStartFrames.append(cd->trk[i - 1].start);
+    cddbInfo.trackInfoList.append(track);
+  }
+  trackStartFrames.append((cd->trk[0]).start);
+  trackStartFrames.append((cd->trk[numberOfTracks]).start);
+
+  populateSongList("");
 }
 
-void KSCD::cddb_no_info()
+void KSCD::cddb_failed(CDDB::Result result)
 {
-    kdDebug(67000) << "cddb_no_info() called\n" << endl;
+    kdDebug(67000) << k_funcinfo << endl;
 
-    tracktitlelist.clear();
-    extlist.clear();
-    tracktitlelist.append(i18n("No matching freedb entry found."));
+    cddbInfo.clear();
+
+    lookupDevice();
+    populateSongList(result == CDDB::NoRecordFound ? i18n("No matching freedb entry found.") : i18n("Error getting freedb entry."));
 
     led_off();
-    
-    get_cdtext_info();
-} // cddb_no_info
-
-void KSCD::cddb_failed()
-{
-    // TODO differentiate between those casees where the communcition really
-    // failed and those where we just couldn't find anything
-    //        cddb_ready_bug = 0;
-    kdDebug(67000) << "cddb_failed() called\n" << endl;
-
-    tracktitlelist.clear();
-    extlist.clear();
-    tracktitlelist.append(i18n("Error getting freedb entry."));
-    revision=year=0;
-    category=genre=QString::null;
-
-    led_off();
-    
-    get_cdtext_info();
 } // cddb_failed
 
 void KSCD::mycddb_inexact_read()
@@ -1478,15 +1484,6 @@ void KSCD::titlelabeltimeout()
 
 } // titlelabeltimeout
 
-void KSCD::setArtist(const QString& artist)
-{
-    if (!artist.isEmpty()) {
-        artistlabel->setText(artist);
-    } else {
-        artistlabel->clear();
-    }
-}
-
 void KSCD::setTitle(int track)
 {
     QString tooltip = artistlabel->text();
@@ -1498,10 +1495,10 @@ void KSCD::setTitle(int track)
         if(track > (int)tracktitlelist.count())
             title = i18n("<Unknown>");
         else
-            title = *tracktitlelist.at(track);
+            title = *tracktitlelist.at(track-1);
 
         titlelabel->setText(title);
-	tooltip += " / ";
+        tooltip += " / ";
         tooltip += KStringHandler::rsqueeze(title, 30);
     }
 
@@ -1627,94 +1624,76 @@ void KSCD::cycletimeout()
 } // cycletimeout
 
 
-bool KSCD::getArtist(QString& artist)
-{
-    if((int)tracktitlelist.isEmpty()){
-        return false;
-    }
-
-    artist = tracktitlelist.first();
-
-    int pos;
-    pos = artist.find('/', 0, true);
-    if(pos != -1)
-        artist.truncate(pos);
-
-    artist = artist.stripWhiteSpace();
-    return true;
-} // getArtist
-
 void KSCD::information(int i)
 {
     //kdDebug(67000) << "Information " << i << "\n" << endl;
 
-    QString artist;
-    QString str;
-
-    if(!getArtist(artist))
+    if(cddbInfo.artist.isEmpty())
         return;
 
-    artist = KURL::encode_string_no_slash(artist);
+    QString encodedArtist = KURL::encode_string_no_slash(cddbInfo.artist);
+
+    QString str;
 
     switch(i)
     {
         case 0:
             str = QString("http://musicmoz.org/cgi-bin/ext.cgi?artist=%1")
-                   .arg(artist);
+                   .arg(encodedArtist);
             break;
 
          case 1:
             str = QString("http://ubl.artistdirect.com/cgi-bin/gx.cgi/AppLogic+Search?select=MusicArtist&searchstr=%1&searchtype=NormalSearch")
-                .arg(artist);
+                .arg(encodedArtist);
             break;
 
         case 2:
             str = QString("http://www.cduniverse.com/cgi-bin/cdubin.exe/rlinka/ean=%1")
-                  .arg(artist);
+                  .arg(encodedArtist);
 
         case 3:
             str = QString("http://www.alltheweb.com/search?cat=web&q=%1")
-                    .arg(artist);
+                    .arg(encodedArtist);
             break;
 
         case 4:
             str = QString("http://altavista.com/web/results?q=%1&kgs=0&kls=1&avkw=xytx")
-                  .arg(artist);
+                  .arg(encodedArtist);
             break;
 
         case 5:
             str = QString("http://msxml.excite.com/_1_2UDOUB70SVHVHR__info.xcite/dog/results?otmpl=dog/webresults.htm&qkw=%1&qcat=web&qk=20&top=1&start=&ver=14060")
-                  .arg(artist);
+                  .arg(encodedArtist);
             break;
 
         case 6:
             str = QString("http://www.google.com/search?q=%1")
-                  .arg(artist);
+                  .arg(encodedArtist);
             break;
 
         case 7:
             str = QString("http://groups.google.com/groups?oi=djq&as_q=%1&num=20")
-                  .arg(artist);
+                  .arg(encodedArtist);
             break;
 
         case 8:
             str = QString("http://www.hotbot.com/default.asp?prov=Inktomi&query=%1&ps=&loc=searchbox&tab=web")
-                  .arg(artist);
+                  .arg(encodedArtist);
             break;
 
         case 9:
             str = QString("http://search.lycos.com/default.asp?lpv=1&loc=searchhp&tab=web&query=%1")
-                  .arg(artist);
+                  .arg(encodedArtist);
              break;
 
          case 10:
              str = QString("http://search.dmoz.org/cgi-bin/search?search=%1")
-                   .arg(artist);
+                   .arg(encodedArtist);
              break;
 
          case 11:
              str = QString("http://search.yahoo.com/bin/search?p=%1")
-                   .arg(artist);
+                   .arg(encodedArtist);
              break;
 
          default:
@@ -1787,37 +1766,43 @@ int KSCD::currentTrack()
     return wm_cd_getcurtrack();
 }
 
+int KSCD::currentTrackLength()
+{
+    if ((wm_cd_status() == WM_CDM_PLAYING) || (wm_cd_status() == WM_CDM_PAUSED))
+    {
+        return cd->trk[wm_cd_getcurtrack() - 1].length;
+    }
+    return -1;
+}
+
+int KSCD::currentPosition()
+{
+    if ((wm_cd_status() == WM_CDM_PLAYING) || (wm_cd_status() == WM_CDM_PAUSED))
+    {
+        return cur_pos_rel;
+    }
+    return -1;
+}
+
+int KSCD::getStatus()
+{
+    return wm_cd_status();
+}
+
 QString KSCD::currentTrackTitle()
 {
     int track = currentTrack();
-    return (track > -1) ? tracktitlelist[track] : QString::null;
+    return (track > -1) ? tracktitlelist[track-1] : QString::null;
 }
 
 QString KSCD::currentAlbum()
 {
-    QString album = tracktitlelist[0];
-    int slash = album.find('/');
-
-    if (slash < 2)
-    {
-        return album;
-    }
-
-
-    return album.right(album.length() - slash - 2);
+  return cddbInfo.title;
 }
 
 QString KSCD::currentArtist()
 {
-    QString artist = tracktitlelist[0];
-    int slash = artist.find('/');
-
-    if (slash < 1)
-    {
-        return artist;
-    }
-
-    return artist.left(slash - 1);
+        return cddbInfo.artist;
 }
 
 QStringList KSCD::trackList()
@@ -1825,25 +1810,27 @@ QStringList KSCD::trackList()
     return tracktitlelist;
 }
 
-void KSCD::clearSongList()
+void KSCD::populateSongList(QString infoStatus)
 {
-    songListCB->clear();
-    QToolTip::remove(songListCB);
-    QToolTip::add(songListCB, i18n("Track list"));
-}
+  // set the artist and title labels as well as the dock tooltip.
+  if (!infoStatus.isEmpty())
+    artistlabel->setText(infoStatus);
+  else
+  if (cddbInfo.artist.isEmpty())
+    artistlabel->setText(cddbInfo.title);
+  else
+  if (cddbInfo.title.isEmpty())
+    artistlabel->setText(cddbInfo.artist);
+  else
+    artistlabel->setText(QString("%1 - %2").arg(cddbInfo.artist, cddbInfo.title));
 
-void KSCD::populateSongList()
-{
-    // need to start i at 0 for the case when tracktitlelist is empty
-    int i = 0;
-    clearSongList();
-    QStringList::Iterator it = tracktitlelist.begin();
-    if(it != tracktitlelist.end() && !(*it).isEmpty())
-        setArtist((*it));
-    else
-        setArtist(i18n("<Unknown>"));
 
-    for (++it; it != tracktitlelist.end(); ++it, ++i )
+  songListCB->clear();
+  QToolTip::remove(songListCB);
+  QToolTip::add(songListCB, i18n("Track list"));
+
+  tracktitlelist.clear();
+  for (int i = 0; i < cddbInfo.trackInfoList.count(); i++)
     {
         int mins = wm_cd_gettracklen(i + 1);
         QString time;
@@ -1854,25 +1841,12 @@ void KSCD::populateSongList()
             time.sprintf(" (%02d:%02d)", mins, secs);
         }
 
-        songListCB->insertItem(QString::fromLocal8Bit("%1: %2%3")
-                               .arg(QString::number(i + 1).rightJustify(2, '0'), *it, time));
-    }
+        songListCB->insertItem(QString("%1: %2%3")
+        .arg(QString::number(i + 1).rightJustify(2, '0'), cddbInfo.trackInfoList[i].title, time));
+    tracktitlelist.append(cddbInfo.trackInfoList[i].title);
+  }
 
-    for(; i < wm_cd_getcountoftracks(); ++i)
-    {
-        int mins = wm_cd_gettracklen(i + 1);
-        QString time;
-        if (mins > 0)
-        {
-            int secs = mins % 60;
-            mins /= 60;
-            time.sprintf(" (%02d:%02d)", mins, secs);
-        }
-        songListCB->insertItem(i18n("%1: <Unknown>%3")
-                               .arg(QString::number(i + 1).rightJustify(2, '0'), time));
-    }
-
-    updateDisplayedTrack(currentTrack());
+  updateDisplayedTrack(currentTrack());
 }
 
 void KSCD::setSongListTo(int cb_index)
@@ -1887,6 +1861,15 @@ void KSCD::setSongListTo(int cb_index)
     QToolTip::add(songListCB, i18n("Current track: %1").arg(justTheName));
 }
 
+static const KCmdLineOptions options[] =
+{
+    {"s",0,0},
+    {"start",I18N_NOOP("Start playing"),0},
+    {"+[device]",I18N_NOOP("CD device, can be a path or a media:/ URL"),0},
+    KCmdLineLastOption
+};
+
+
 /**
  * main()
  */
@@ -1907,13 +1890,28 @@ int main( int argc, char *argv[] )
     aboutData.addCredit("freedb.org", I18N_NOOP("Special thanks to freedb.org for providing a free CDDB-like CD database"), 0, "http://freedb.org");
 
     KCmdLineArgs::init( argc, argv, &aboutData );
-
+    KCmdLineArgs::addCmdLineOptions(options);
     KUniqueApplication::addCmdLineOptions();
 
+    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
     if (!KUniqueApplication::start())
     {
-        fprintf(stderr, "kscd is already running!\n");
-        exit(0);
+      fprintf(stderr, "kscd is already running!\n");
+      if (args->count()>0 || args->isSet("start"))
+      {
+        DCOPClient cli;
+        if (!cli.attach()) exit(0);
+        DCOPRef ref("kscd","CDPlayer");
+        if (args->count()>0)
+        {
+          ref.send("setDevice(QString)",QString(args->arg(0)));
+        }
+        if (args->isSet("start"))
+        {
+          ref.send("play()");
+        }
+      }
+      exit(0);
     }
 
     KUniqueApplication a;
@@ -1939,6 +1937,9 @@ int main( int argc, char *argv[] )
         k->show();
     }
 
+    if (args->count()>0) Prefs::self()->setCdDevice(args->arg(0));
+    if (args->isSet("start")) Prefs::self()->setAutoplay(true);
+
     return a.exec();
 } // main()
 
