@@ -23,13 +23,14 @@
 
 #include <kdebug.h>
 #include <kio/job.h>
+#include <kio/netaccess.h>
 
 #include "synchttplookup.h"
 
 namespace KCDDB
 {
   SyncHTTPLookup::SyncHTTPLookup()
-    : HTTPLookup(), done_( false )
+    : HTTPLookup()
   {
   }
 
@@ -81,7 +82,6 @@ namespace KCDDB
     CDDB::Result
   SyncHTTPLookup::runQuery()
   {
-    done_ = false;
     data_ = QByteArray();
     state_ = WaitingForQueryResponse;
 
@@ -90,9 +90,6 @@ namespace KCDDB
     if ( Success != result_ )
       return result_;
 
-    while ( !done_ )
-      qApp->processOneEvent();
-
     kdDebug(60010) << "runQuery() Result: " << resultToString(result_) << endl;
 
     return result_;
@@ -101,7 +98,6 @@ namespace KCDDB
     CDDB::Result
   SyncHTTPLookup::matchToCDInfo( const CDDBMatch & match )
   {
-    done_ = false;
     data_ = QByteArray();
     state_ = WaitingForReadResponse;
 
@@ -110,18 +106,25 @@ namespace KCDDB
     if ( Success != result_ )
       return result_;
 
-    while ( !done_ )
-      qApp->processOneEvent();
-
     return result_;
   }
 
-    void
-  SyncHTTPLookup::slotResult(  KIO::Job *job )
+    CDDB::Result
+  SyncHTTPLookup::fetchURL()
   {
-    done_ = true;
+    kdDebug(60010) << "About to fetch: " << cgiURL_.url() << endl;
 
-    HTTPLookup::slotResult( job );
+    KIO::TransferJob* job = KIO::get( cgiURL_, false, false );
+
+    if ( 0 == job )
+      return ServerError;
+
+    if (!KIO::NetAccess::synchronousRun(job, 0, &data_))
+      return ServerError;
+
+    slotResult(job);
+
+    return Success;
   }
 }
 
