@@ -1,5 +1,5 @@
 /*
- * system dependend wrapper for access to cdrom
+ * system dependent wrapper for access to cdrom
  * Copyright (C) 1999  Martin Vogt
  *
  * This program is free software; you can redistribute it and/or modify
@@ -8,7 +8,7 @@
  *
  *  For more information look at the file COPYRIGHT in this package
  *
- * $Id: cdromAccess.cpp,v 1.3 2000/11/23 18:43:20 mvogt Exp $
+ * $Id: cdromAccess.cpp,v 1.10 2004/12/16 21:36:44 cartman Exp $
  *
  */
 
@@ -23,6 +23,18 @@
 #endif
 
 #ifdef OS_Linux
+  #include <linux/version.h>
+  #include <sys/types.h>
+  #ifndef __u64
+  #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,70)
+  #include <bits/wordsize.h>
+  #if __WORDSIZE == 64
+  typedef unsigned long __u64;
+  #else
+  typedef unsigned long long __u64;
+  #endif 
+  #endif
+  #endif
   #include "cdromAccess_Linux.cpp"
 #endif
 
